var spawn   = require("child_process").spawn;
var path    = require("path");
var fs      = require("fs");
var fse     = require("fs-extra");
var temp    = require("temp");
var through = require("through");

//Eagerly create temporary directory
var directory_built = false
  , directory_err   = null
  , directory_wait  = []
  , directory_path  = "/tmp"
  , directory_count = 0;
temp.mkdir("node-dvisvgm", function(err, dirpath) {
  if(!err) {
    process.on("exit", function() {
      fse.removeSync(dirpath);
    });
  }
  directory_err = err;
  directory_path = dirpath;
  directory_built = true;
  for(var i=0; i<directory_wait.length; ++i) {
    directory_wait[i]();
  }
  directory_wait.length = 0;
});

//Waits for directory to be built
function awaitDir(cb) {
  function makeLocalDir() {
    if(directory_err) {
      cb(directory_err, null);
      return;
    }
    var temp_path = path.join(directory_path, "" + directory_count++);
    console.log("temp_path: "+temp_path);
    fse.mkdirp(temp_path, function(err) {
      if(err) {
        cb(err, null);
        return;
      }
      cb(null, temp_path);
    });
  }
  if(directory_built) {
    makeLocalDir();
  } else {
    directory_wait.push(makeLocalDir);
  }
}

//Send errors downstream to result
function handleErrors(dirpath, result) {
  var log_file = path.join(dirpath, "dvisvgm.log");
  fs.exists(log_file, function(exists) {
    if(!exists) {
      fse.remove(dirpath);
      result.emit("error", new Error("Error running dvisvgm"));
      return;
    }
    //Try to crawl through the horrible mess that LaTeX shat upon us
    var log = fs.createReadStream(log_file);
    var err = [];
    log.on("data", function(data) {
      var lines = data.toString().split("\n");
      for(var i=0; i<lines.length; ++i) {
        var l = lines[i];
        if(l.length > 0 && l.charAt(0) === "!") {
          err.push(lines[i]);
        }
      }
    });
    log.on("end", function() {
      if(err.length > 0) {
        err.unshift("dvisvgm Syntax Error");
        result.emit("error", new Error(err.join("\n")));
      } else {
        result.emit("error", new Error("Unspecified dvisvgm error"));
      }
    });
  });
}

//Converts a expression into a LaTeX image
module.exports = function(doc, options) {
  if(!options) {
    options = {};
  }
  
  //var format = options.format || "pdf";
  
  //LaTeX command
  //var tex_command = options.command || (format === "pdf" ? "pdflatex" : "latex");
  
  var dvi_command = "dvisvgm";
  
  //Create result
  var result = through();
  awaitDir(function(err, dirpath) {
    function error(e) {
      result.emit("error", e);
      result.destroySoon();
    }
    if(err) {
      error(err);
      return;
    }
    //Write data to tex file
    console.log("dirpath: "+dirpath);
    if(fs.existsSync(dirpath)) {
        console.log("dirpath exists");
    } else {
        console.log("dirpath does not exist");
    }
    var input_path = path.join(dirpath, "dvisvgmput.dvi");
    var dvi_file = fs.createWriteStream(input_path);
    debugger; 
    dvi_file.on("close", function() {
      //Invoke LaTeX
      debugger;
      var dvi = spawn(dvi_command, [
        "dvisvgmput.dvi"
      ], {
        cwd: dirpath,
        env: process.env//,
        //stdio: ['ignore','ignore','ignore']
      });

      dvi.on('data', (data) => {
        console.log(data);
      });


      // Let the user know if LaTeX couldn't be found
      dvi.on('error', function(err) {
        if (err.code === 'ENOENT') {
          console.log(err);
          console.error("\nThere was an error spawning " + dvi_command + ". \n"
                        + "Please make sure your LaTeX distribution is"
                        + "properly installed.\n");
        }
      });

      //Wait for LaTeX to finish its thing
      dvi.on("exit", function(code, signal) {
        var output_file = path.join(dirpath, "dvisvgmput.svg");
        fs.exists(output_file, function(exists) {
          if(exists) {
            var stream = fs.createReadStream(output_file);
            stream.on("close", function() {
              // TODO remove dirapth
              fse.remove(dirpath);
            });
            stream.pipe(result);
          } else {
            handleErrors(dirpath, result);
          }
        });
      });
    });
    
    debugger;
    if(typeof doc === "string" || doc instanceof Buffer) {
      dvi_file.end(doc);
    } else if(doc instanceof Array) {
      for(var i=0; i<doc.length; ++i) {
        dvi_file.write(doc[i]);
      }
      dvi_file.end();
    } else if(doc.pipe) {
      doc.pipe(dvi_file);
    } else {
      error(new Error("Invalid document"));
      return;
    }
  });
  
  return result;
}

